%%Import data Problems 1-3 solution

close all
clear
 
filename = 'GrayBootCamp2012__mm9-(Total1,Bulent)(0hr,1hr,6hr)__(EXN,INT)-EXPRESSION.txt';

%% importdata
 
% The function importdata will load the file as a struct with two fields: 
% data and textdata. 
myStruct = importdata(filename);

% The textdata field is a cell array except for the contiguous set of
% columns that Matlab thinks is numeric data. Columns 9 through 36 have 
% only numeric data, so these columns are in S.data. Other columns like
% "Start" and "Stop" have numeric data, but they are in S.textdata.

%% Problem 1: extract headers

% We want the first row of S.textdata, since this contains the column names
% of headers.
headers = myStruct.textdata(1,:);

%% Problem 2: Use headers to get gene names

% We use the headers array to find the column that contains the gene names
% The strcmpi function will return a logical array with true values where
% it finds the string 'Gene'
geneColumn = strcmpi(headers,'Gene');

% We use this logical array as an index to extract the gene names. We also
% want top ignore the first row that has the word 'Gene', so we extract
% rows 2:end
geneNames = myStruct.textdata(2:end,geneColumn);

%% Problem 3: Numeric header

% It's also useful to have a separate header for the numeric data, so we
% can use it to find specific columns in the data array

% We can again use strcmp but not look for the empty columns in the 2nd row
dataColumns = strcmpi(myStruct.textdata(2,:),'');

% Extract the headers for these columns
dataHeaders = myStruct.textdata(1,dataColumns);

%% Why did we do this?

% You've already seen why its useful to extract the headers. We used it to
% find the gene column. 

% We can similarly search the dataHeaders to find a column of numeric data.
% For example, say we want the column named 'Exn_nrds.Rep1.0hr'. 

expColumn = strcmpi(dataHeaders,'Exn_nrds.Rep1.0hr');

% This has a single 1 in the 5th spot, and the column we want in S.data is
% the 5th column. 
expData = myStruct.data(:,expColumn);

% Now we can do whatever we want with this data, like a histogram
figure
hist(log2(expData))

%% Why not use tables?

% The short answer is that Tables in Matlab are relatively new, so there's 
% still a lot of code out there that uses functions like importata or 
% xlsread, which load files in a weird way.

% But if we want to use tables, we can use the function readtable. Since
% the information in tab-separated, we specify a tab as the delimiter
myTable = readtable(filename,'Delimiter','\t');

%% Why tables allow us to skip doing problems 1-3

% We don't actually need to extract the headers as a separate variable.
% We previously did it to find the gene name column, knowing that it was 
% named gene. We can do this more simply now:

geneNames = myTable.Gene;

% Or, we could write;

geneNames = myTable{:,'Gene'};

% Notice that if you find columns this way, you must use the curly brackets 
% to get the actual data. Using parentheses, e.g. T(:.'Gene') would return 
% a Table with a single column.

%% What if we still want the headers?

% There are two ways to do this:

headers = fieldnames(myTable);

% or

headers = myTable.Properties.VariableNames;

%One thing to notice here is that Matlab replaced the dots in the names
%with underscores. So 'Exn_nrds.Rep1.0hr' became 'Exn_nrds_Rep1_0hr'

%% Why do we need to know the column names? We can just look at the data and say, I want column 4

% Using the column names like 'Exn_nrds.Rep1.0hr' is more robust because
% you can automate ways of extracting the data.

% For example, say you want to use the plotmatrix function to look at how
% some subset of the columns covary. You can specify the things you want in
% variables and then use loops to specify the column names

features = {'EXN_nrds','INT_nrds'};
replicates = [1,2];   
times = [0 6]; 

% Loop through these desired variables, making strings for the column names
columnNames = {};
for iFeat = features
    for iRep = replicates
        for iTime = times
            columnNames{end+1} = [iFeat{1} '_Rep' num2str(iRep) '_' num2str(iTime) 'hr']; 
        end
    end
end
    

% Now we extract the data, log transform, and call plotmatrix, all in one
% line of code
figure, plotmatrix(log2(myTable{:,columnNames}));

% Neat, huh? 
